/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.myfaces.orchestra.urlParamNav;

import javax.faces.application.NavigationHandler;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;
import java.io.IOException;

/**
 * This navigation handler replaces EL expressions (value bindings) within the to-view-id
 * property of a navigation rule.
 * <p>
 * Since we do not know quite how this feature may interact with other JSF frameworks that
 * also provide navigation handlers, this feature is not enabled by default.
 * <p>
 * To enable this feature:
 * <ul>
 * <li>add this navigation handler to your faces-config.xml</li>
 * <li>add the {@link UrlParameterViewHandler} to your faces-config.xml</li>
 * <li>configure the navigation case to use &lt;redirect/&gt; flag.<li/>
 * </ul>
 * Note that redirect is required for the rule; this is only natural as without redirect
 * there is no bookmarkable URL for the user. 
 */
public class UrlParameterNavigationHandler extends NavigationHandler
{
	private final NavigationHandler original;

	public UrlParameterNavigationHandler(final NavigationHandler original)
	{
		this.original = original;
	}

	public void handleNavigation(final FacesContext context, String fromAction, String outcome)
	{
		original.handleNavigation(new FacesContextWrapper(context)
		{
			public ExternalContext getExternalContext()
			{
				return new ExternalContextWrapper(super.getExternalContext())
				{
					public void redirect(String url) throws IOException
					{
						super.redirect(interceptRedirect(context, url));
					}
				};
			}
		}, fromAction, outcome);
	}

	// TODO: this only supports one EL expression at the moment; it would be nice
	// to support multiple.
	protected String interceptRedirect(FacesContext context, String url)
	{
		int pos = url.indexOf("#{");
		if (pos > -1 && url.indexOf("}", pos) > -1)
		{
			ValueBinding vb = context.getApplication().createValueBinding(url);
			return (String) vb.getValue(context);
		}

		return url;
	}
}
