/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.myfaces.orchestra.conversation;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.Arrays;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

/**
 * A ConversationContext is a container for a set of conversations.
 * <p>
 * Normally there is only one ConversationContext per http session. However there can
 * be multiple instances if the user has multiple concurrent windows open into the same
 * webapp, using the ox:separateConversationContext or other similar mechanism.
 * <p>
 * Like the conversation class, a context can also have a timeout which will cause it
 * to be ended automatically if not accessed within the given period.
 */
public class ConversationContext
{
	private final Log log = LogFactory.getLog(ConversationContext.class);

	// This id is attached as a query parameter to every url rendered in a page
	// (forms and links) so that if that url is invoked then the request will
	// cause the same context to be used from the user's http session. 
	private final long id;

	// See addAttribute
	private final Map attributes = new TreeMap();

	// The conversations held by this context, keyed by conversation name.
	private final Map conversations = new TreeMap();

	// time at which this was last accessed, used for timeouts.
	private long lastAccess;

	// default timeout for contexts: 30 minutes.
	private long timeoutMillis = 30 * 60 * 1000;

	protected ConversationContext(long id)
	{
		this.id = id;

		touch();
	}

	/**
	 * The conversation context id, unique within the current http session.
	 */
	public long getId()
	{
		return id;
	}

	/**
	 * Mark this context as having been used.
	 */
	protected void touch()
	{
		lastAccess = System.currentTimeMillis();
	}

	/**
	 * The system time in millis when this conversation has been accessed last.
	 */
	public long getLastAccess()
	{
		return lastAccess;
	}

	/**
	 * Get the timeout after which this context will be closed.
	 *
	 * @see #setTimeout
	 */
	public long getTimeout()
	{
		return timeoutMillis;
	}

	/**
	 * Set the timeout after which this context will be closed.
	 * <p>
	 * A value of -1 means no timeout checking.
	 */
	public void setTimeout(long timeoutMillis)
	{
		this.timeoutMillis = timeoutMillis;
	}

	/**
	 * Invalidate all conversations within this context.
	 */
	protected void clear()
	{
		synchronized (this)
		{
			Conversation[] convArray = new Conversation[conversations.size()];
			conversations.values().toArray(convArray);

			for (int i = 0; i < convArray.length; i++)
			{
				Conversation conversation = convArray[i];
				conversation.invalidate();
			}

			conversations.clear();
		}
	}

	/**
	 * Start a conversation if not already started.
	 */
	protected Conversation startConversation(String name, ConversationFactory factory)
	{
		synchronized (this)
		{
			touch();
			Conversation conversation = (Conversation) conversations.get(name);
			if (conversation == null)
			{
				conversation = factory.createConversation(this, name);

				conversations.put(name, conversation);
			}
			return conversation;
		}
	}

	/**
	 * Remove the conversation from this context.
	 * 
	 * <p>Notice: It is assumed that the conversation has already been invalidated.</p>
	 */
	protected void removeConversation(Conversation conversation)
	{
		synchronized (this)
		{
			touch();
			conversations.remove(conversation.getName());
		}
	}

	/**
	 * Remove the conversation with the given name from this context.
	 * 
	 * <p>Notice: Its assumed that the conversation has already been invalidated</p>
	 */
	protected void removeConversation(String name)
	{
		synchronized (this)
		{
			touch();
			Conversation conversation = (Conversation) conversations.get(name);
			if (conversation != null)
			{
				removeConversation(conversation);
			}
		}
	}

	/**
	 * See if there is a conversation with the specified name.
	 */
	protected boolean hasConversations()
	{
		synchronized (this)
		{
			touch();
			return conversations.size() > 0;
		}
	}

	/**
	 * Check if the given conversation exists.
	 */
	protected boolean hasConversation(String name)
	{
		synchronized (this)
		{
			touch();
			return conversations.get(name) != null;
		}
	}

	/**
	 * Get a conversation by name.
	 */
	protected Conversation getConversation(String name)
	{
		synchronized (this)
		{
			touch();

			Conversation conv = (Conversation) conversations.get(name);
			if (conv != null)
			{
				conv.touch();
			}

			return conv;
		}
	}

	/**
	 * Iterates over all the conversations in this context.
	 *
	 * @return An iterator over a copy of the conversation list. It is safe to remove objects from
	 * the conversation list while iterating, as the iterator refers to a different collection.
	 */
	public Iterator iterateConversations()
	{
		synchronized (this)
		{
			touch();

			Conversation[] convs = (Conversation[]) conversations.values().toArray(new Conversation[conversations.size()]);
			return Arrays.asList(convs).iterator();
		}
	}

	/**
	 * Check the timeout for every conversation in this context.
	 * <p>
	 * This method does not check the timeout for this context object itself.
	 */
	protected void checkConversationTimeout()
	{
		synchronized (this)
		{
			Conversation[] convArray = new Conversation[conversations.size()];
			conversations.values().toArray(convArray);

			for (int i = 0; i < convArray.length; i++)
			{
				Conversation conversation = convArray[i];

				ConversationTimeoutableAspect timeoutAspect =
					(ConversationTimeoutableAspect)
						conversation.getAspect(ConversationTimeoutableAspect.class);

				if (timeoutAspect != null && timeoutAspect.isTimeoutReached())
				{
					if (log.isDebugEnabled())
					{
						log.debug("end conversation due to timeout: " + conversation.getName());
					}

					conversation.invalidate();
				}
			}
		}
	}

	/**
	 * Add an attribute to the conversationContext.
	 * <p>
	 * A context provides a map into which any arbitrary objects can be stored. It
	 * isn't a major feature of the context, but can occasionally be useful.
	 */
	public void setAttribute(String name, Object attribute)
	{
		synchronized(attributes)
		{
			attributes.remove(name);
			attributes.put(name, attribute);
		}
	}

	/**
	 * Check if this conversationContext holds a specific attribute.
	 */
	public boolean hasAttribute(String name)
	{
		synchronized(attributes)
		{
			return attributes.containsKey(name);
		}
	}

	/**
	 * Get a specific attribute.
	 */
	public Object getAttribute(String name)
	{
		synchronized(attributes)
		{
			return attributes.get(name);
		}
	}

	/**
	 * Remove an attribute from the conversationContext.
	 */
	public Object removeAttribute(String name)
	{
		synchronized(attributes)
		{
			return attributes.remove(name);
		}
	}
}