/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.myfaces.orchestra.conversation;

import java.io.IOException;
import java.io.ObjectStreamException;
import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.myfaces.orchestra.frameworkAdapter.FrameworkAdapter;
import org.apache.myfaces.orchestra.requestParameterProvider.RequestParameterProviderManager;

/**
 * Deals with the various conversation contexts in the current session.
 * <p>
 * A new conversation context will be created if the servlet request did
 * not specify an existing conversation context id.
 * <p>
 * At the current time, this object does not serialize well. Any attempt to serialize
 * this object (including any serialization of the user session) will just cause it
 * to be discarded.
 * <p>
 * TODO: fix serialization issues.
 */
public class ConversationManager implements Serializable
{
	final static String CONVERSATION_CONTEXT_PARAM = "conversationContext";

	private final static String CONVERSATION_MANAGER_KEY = "org.apache.myfaces.ConversationManager";
	private final static String CONVERSATION_CONTEXT_REQ = "org.apache.myfaces.ConversationManager.conversationContext";

	private static final Iterator EMPTY_ITERATOR = Collections.EMPTY_LIST.iterator();

	// TODO: consider whether a static value is the best way to generate context ids; they
	// only need to be unique within a user session. Statics are also test-unfriendly.
	private static long nextConversationContextId = 1;

	private final Log log = LogFactory.getLog(ConversationManager.class);

	// This member must always be accessed with a lock held on the parent ConverstationManager instance;
	// a HashMap is not thread-safe and this class must be thread-safe.
	private final Map conversationContexts = new HashMap();

	protected ConversationManager()
	{
		// conversationMessager = createMessager();
	}

	/**
	 * Get the conversation manager. This creates a new one if none exists.
	 */
	public static ConversationManager getInstance()
	{
		return getInstance(true);
	}

	/**
	 * Get the conversation manager.
	 * <p>
	 * When create is true, an instance is always returned; one is
	 * created if none currently exists for the current user session.
	 * <p>
	 * When create is false, null is returned if no instance yet
	 * exists for the current user session.
	 */
	public static ConversationManager getInstance(boolean create)
	{
		ConversationManager conversationManager = (ConversationManager) FrameworkAdapter.getCurrentInstance().getSessionAttribute(CONVERSATION_MANAGER_KEY);
		if (conversationManager == null && create)
		{
			// TODO: do not call new directly here, as it makes it impossible to configure
			// an alternative ConversationManager instance. This is IOC and test unfriendly.
			conversationManager = new ConversationManager();

			// initialize environmental systems
			RequestParameterProviderManager.getInstance().register(new ConversationRequestParameterProvider());

			// set mark
			FrameworkAdapter.getCurrentInstance().setSessionAttribute(CONVERSATION_MANAGER_KEY, conversationManager);
		}

		return conversationManager;
	}

	/**
	 * Get the current, or create a new unique conversationContextId.
	 * <p>
	 * The current conversationContextId will retrieved from the request parameters. If no such parameter is
	 * present then a new id will be allocated and a new ConversationContext created.
	 * <p>
	 * In either case the result will be stored within the request for faster lookup.
	 * <p>
	 * Note that there is no security flaw regarding injection of fake context ids; the id must match one already
	 * in the session and there is no security problem with two windows in the same session exchanging ids.
	 */
	public Long getConversationContextId()
	{
		Long conversationContextId = (Long) FrameworkAdapter.getCurrentInstance().getRequestAttribute(CONVERSATION_CONTEXT_REQ);
		if (conversationContextId == null)
		{
			if (FrameworkAdapter.getCurrentInstance().containsRequestParameterAttribute(CONVERSATION_CONTEXT_PARAM))
			{
				String urlConversationContextId = FrameworkAdapter.getCurrentInstance().getRequestParameterAttribute(CONVERSATION_CONTEXT_PARAM).toString();
				conversationContextId = new Long(Long.parseLong(urlConversationContextId, Character.MAX_RADIX));
			}
			else
			{
				synchronized (ConversationManager.class)
				{
					conversationContextId = new Long(nextConversationContextId);
					nextConversationContextId++;
				}
			}

			FrameworkAdapter.getCurrentInstance().setRequestAttribute(CONVERSATION_CONTEXT_REQ, conversationContextId);
		}

		return conversationContextId;
	}

	/**
	 * Get the conversation context for the given id
	 */
	protected ConversationContext getConversationContext(Long conversationContextId)
	{
		synchronized (this)
		{
			return (ConversationContext) conversationContexts.get(conversationContextId);
		}
	}

	/**
	 * Get the conversation context for the given id. <br />
	 * If there is no conversation context a new one will be created
	 */
	protected ConversationContext getOrCreateConversationContext(Long conversationContextId)
	{
		synchronized (this)
		{
			ConversationContext conversationContext = (ConversationContext) conversationContexts.get(conversationContextId);
			if (conversationContext == null)
			{
				conversationContext = new ConversationContext(conversationContextId.longValue());
				conversationContexts.put(conversationContextId, conversationContext);
			}

			return conversationContext;
		}
	}

	/**
	 * Ends all conversations within the current context; the context itself will remain active.
	 */
	public void clearCurrentConversationContext()
	{
		Long conversationContextId = getConversationContextId();
		if (conversationContextId == null)
		{
			return;
		}

		ConversationContext conversationContext = getConversationContext(conversationContextId);
		if (conversationContext != null)
		{
			conversationContext.clear();
		}
	}

	/**
	 * <p>Destroy the given conversation context</p>
	 * <p/>
	 * <p>Notice: its assumed that the context is already been destroyed</p>
	 */
	protected void removeConversationContext(Long conversationContextId)
	{
		synchronized (this)
		{
			conversationContexts.remove(conversationContextId);
		}
	}

	/**
	 * Start a conversation.
	 *
	 * @see ConversationContext#startConversation(String, ConversationFactory)
	 */
	public Conversation startConversation(String name, ConversationFactory factory)
	{
		Long conversationContextId = getConversationContextId();
		ConversationContext conversationContext = getOrCreateConversationContext(conversationContextId);
		return conversationContext.startConversation(name, factory);
	}

	/**
	 * Remove a conversation
	 *
	 * <p>Notice: Its assumed that the conversation has already been invalidated</p>
	 *
	 * @see ConversationContext#removeConversation(String)
	 */
	protected void removeConversation(String name)
	{
		Long conversationContextId = getConversationContextId();
		ConversationContext conversationContext = getConversationContext(conversationContextId);
		if (conversationContext != null)
		{
			conversationContext.removeConversation(name);
		}
	}

	/**
	 * Get the conversation with the given name
	 *
	 * @return null if no conversation context is active or if the conversation did not exist.
	 */
	public Conversation getConversation(String name)
	{
		ConversationContext conversationContext = getCurrentConversationContext();
		if (conversationContext == null)
		{
			return null;
		}
		return conversationContext.getConversation(name);
	}

	/**
	 * check if the given conversation is active
	 */
	public boolean hasConversation(String name)
	{
		ConversationContext conversationContext = getCurrentConversationContext();
		if (conversationContext == null)
		{
			return false;
		}
		return conversationContext.hasConversation(name);
	}

	/**
	 * Returns an iterator over all the Conversation objects in the current conversation
	 * context. Never returns null, even if no conversation context exists.
	 */
	public Iterator iterateConversations()
	{
		ConversationContext conversationContext = getCurrentConversationContext();
		if (conversationContext == null)
		{
			return EMPTY_ITERATOR;
		}

		return conversationContext.iterateConversations();
	}

	/**
	 * Get the current conversation context.
	 *
	 * @return null if there is no context active
	 */
	public ConversationContext getCurrentConversationContext()
	{
		Long conversationContextId = getConversationContextId();
		if (conversationContextId == null)
		{
			return null;
		}

		ConversationContext conversationContext = getConversationContext(conversationContextId);
		return conversationContext;
	}

	/**
	 * check if we have a conversation context
	 */
	public boolean hasConversationContext()
	{
		return
			(
				FrameworkAdapter.getCurrentInstance().containsRequestAttribute(CONVERSATION_CONTEXT_REQ) ||
					FrameworkAdapter.getCurrentInstance().containsRequestParameterAttribute(CONVERSATION_CONTEXT_REQ)) &&
				getCurrentConversationContext() != null;
	}

	/**
	 * Get the Messager used to inform the user about anomalies.
	 * <p>
	 * What instance is returned is controlled by the FrameworkAdapter. See 
	 * {@link org.apache.myfaces.orchestra.frameworkAdapter.FrameworkAdapter} for details.
	 */
	public ConversationMessager getMessager()
	{
		return FrameworkAdapter.getCurrentInstance().getConversationMessager();
	}

	/**
	 * Check the timeout for each conversation context, and all conversations
	 * within those contexts.
	 * <p>
	 * If any conversation has not been accessed within its timeout period
	 * then clear the context.
	 * <p>
	 * Invoke the checkTimeout method on each context so that any conversation
	 * that has not been accessed within its timeout is invalidated.
	 */
	protected void checkTimeouts()
	{
		Map.Entry[] contexts;
		synchronized (this)
		{
			contexts = new Map.Entry[conversationContexts.size()];
			conversationContexts.entrySet().toArray(contexts);
		}

		long checkTime = System.currentTimeMillis();

		for (int i = 0; i<contexts.length; i++)
		{
			Map.Entry context = contexts[i];

			Long conversationContextId = (Long) context.getKey();
			ConversationContext conversationContext = (ConversationContext) context.getValue();
			conversationContext.checkConversationTimeout();

			if (conversationContext.getTimeout() > -1 &&
				(conversationContext.getLastAccess() +
				conversationContext.getTimeout()) < checkTime)
			{
				if (log.isDebugEnabled())
				{
					log.debug("end conversation context due to timeout: " + conversationContext.getId());
				}

				conversationContext.clear();
				synchronized (this)
				{
					conversationContexts.remove(conversationContextId);
				}
			}
		}
	}

	/*
	 * check if a bean with the given name will be managed by the conversationManager.
	public boolean isManagedBean(String beanName)
	{
		ConfigurableApplicationContext applicationContext = FrameworkAdapter.getInstance().getApplicationContext();
		ConfigurableListableBeanFactory beanFactory = applicationContext.getBeanFactory();

		// XXX: Hack to get the real definition in case of scoped targets
		BeanDefinition beanDefinition = beanFactory.getBeanDefinition("scopedTarget." + beanName);
		if (beanDefinition == null)
		{
			beanDefinition = beanFactory.getBeanDefinition(beanName);
		}

		return beanDefinition.getScope() != null && managedScopes.contains(beanDefinition.getScope());
	}
	 */

	private void writeObject(java.io.ObjectOutputStream out) throws IOException
	{
		// the conversation manager is not (yet) serializable, we just implement it
		// to make it work with distributed sessions
	}

	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException
	{
		// nothing written, so nothing to read
	}

	private Object readResolve() throws ObjectStreamException
	{
		// do not return a real object, that way on first request a new conversation manager will be created
		return null;
	}
}
