/*
 * Copyright (c) 2007, Your Corporation. All Rights Reserved.
 */

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.myfaces.orchestra.conversation;

import org.apache.myfaces.orchestra.frameworkAdapter.FrameworkAdapter;

import java.io.IOException;

/**
 * Some helpers usable for public use
 */
public final class ConversationUtils
{
	private ConversationUtils()
	{
	}

	/**
	 * End and restart the given conversation.
	 * <p>
	 * In contrast to {@link Conversation#invalidateAndRestart()} this
	 * method returns a new instance of the "controller bean" for the given conversation.
	 * <p>
	 * The "controller bean" is the one you configured in your dependency injection framework.
	 */
	public static Object invalidateAndRestart(Conversation conversation)
	{
		String name = conversation.getName();

		conversation.invalidateAndRestart();

		return FrameworkAdapter.getCurrentInstance().getBean(name);
	}

	/**
	 * End and restart the current conversation.
	 * <p>
	 * In contrast to {@link #invalidateAndRestart(Conversation)} this
	 * method returns a new instance of the "current bean".
	 */
	public static Object invalidateAndRestartCurrent()
	{
		CurrentConversationInfo currentConversationInfo = Conversation.getCurrentInstanceInfo();
		String name = currentConversationInfo.getBeanName();

		currentConversationInfo.getConversation().invalidateAndRestart();

		return FrameworkAdapter.getCurrentInstance().getBean(name);
	}

	/**
	 * If no conversation with name <code>conversationName</code> is active a redirect to
	 * <code>redirectViewId</code> will be issued.
	 * <p>
	 * If <code>redirectViewId</code> starts with an slash ('/') the context path will be added.
	 */
	public static void ensureConversationRedirect(String conversationName, String redirectViewId)
	{
		if (!ConversationManager.getInstance().hasConversation(conversationName))
		{
			try
			{
				FrameworkAdapter.getCurrentInstance().redirect(redirectViewId);
			}
			catch (IOException e)
			{
				throw new RuntimeException(e);
			}
		}
	}

	/**
	 * Invalidates a conversation if it exists.
	 */
	public static void invalidateIfExists(String name)
	{
		Conversation conversation = ConversationManager.getInstance().getConversation(name);
		if (conversation != null)
		{
			conversation.invalidate();
		}
	}
}
