/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.myfaces.orchestra.conversation;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.HashMap;
import java.util.Map;

/**
 * The ConversationWiperThread will trigger the conversation timeout check.
 * <p>
 * This is typically started from a servlet session listener when the webapp starts.
 * 
 * @see org.apache.myfaces.orchestra.servlet.ConversationManagerSessionListener.
 */
public class ConversationWiperThread extends Thread
{
	private final Log log = LogFactory.getLog(ConversationWiperThread.class);

	private final long checkTime;

	private Map conversationManagers = new HashMap();

	/**
	 * Constructor.
	 * 
	 * @param checkTime is interval in milliseconds between scans of the existing
	 * ConversationManagers to look for timeouts.
	 */
	public ConversationWiperThread(long checkTime)
	{
		this.checkTime = checkTime;

		setDaemon(true);
		setName(ConversationWiperThread.class.getName());
	}

	/**
	 * Add a ConversationManager to check.
	 * <p>
	 * If there is already a ConversationManager associated with the given
	 * <code>id</code>, the old ConversationManager will be replaced.
	 */
	public void addConversationManager(String id, ConversationManager conversationManager)
	{
		synchronized (conversationManagers)
		{
			conversationManagers.put(id, conversationManager);
		}
	}

	/**
	 * Remove a ConversationManager from the list to check.
	 */
	public void removeConversationManager(String id)
	{
		synchronized (conversationManagers)
		{
			conversationManagers.remove(id);
		}
	}

	public void run()
	{
		if (log.isDebugEnabled())
		{
			log.debug("ConversationWiperThread startup"); // NON-NLS
		}
		_run();
		if (log.isInfoEnabled())
		{
			log.debug("ConversationWiperThread shtudown"); // NON-NLS
		}
	}

	private void _run()
	{
		while (!isInterrupted())
		{
			ConversationManager[] managersArray;
			synchronized (conversationManagers)
			{
				managersArray = new ConversationManager[conversationManagers.size()];
				conversationManagers.values().toArray(managersArray);
			}

			for (int i = 0; i<managersArray.length; i++)
			{
				ConversationManager conversationManager = managersArray[i];
				conversationManager.checkTimeouts();
			}

			try
			{
				Thread.sleep(checkTime);
			}
			catch (InterruptedException e)
			{
				return;
			}
		}
	}
}
