/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.myfaces.orchestra.viewController;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

/**
 * <p>Map view-ids to bean names.</p>
 * <p/>
 * The strategy of this mapper is as follows:
 * <ul>
 * <li>slash ('/') will be removed</li>
 * <li>every character after a slash will be convertert to uppercase, except the first char</li>
 * <li>everyting after a dot ('.') will be removed</li>
 * <li>reserved words {@link #RESERVED_WORDS} are prefixed with an underscore ('_')</li>
 * <li>result starting with an digit are prefixed too </li>
 * </ul>
 * </p>
 * <p/>
 * Examples:
 * <table border="1">
 * <thead><tr><th>ViewId</th><th>BeanName</th></tr></thead>
 * <tbody>
 * <tr><td>/userInfo.jsp</td><td>userInfo</td></tr>
 * <tr><td>/SecureArea/userPassword.jsp</td><td>secureAreaUserPassword</td></tr>
 * </tbody>
 * </table>
 * </p>
 */
public class DefaultViewControllerNameMapper implements ViewControllerNameMapper
{
	private static Set RESERVED_WORDS = new HashSet(Arrays.asList(
		new String[]
			{
				"applicationScope",
				"cookie",
				"facesContext",
				"header",
				"headerValues",
				"initParam",
				"param",
				"paramValues",
				"requestScope",
				"sessionScope",
				"view"
			}
	));


	public String mapViewId(String viewId)
	{
		if (viewId == null)
		{
			return null;
		}

		boolean nextUpper = false;

		StringBuffer sb = new StringBuffer(viewId);
		for (int i = 0; i < sb.length(); i++)
		{
			char c = sb.charAt(i);
			if (c == '/')
			{
				if (i > 0)
				{
					nextUpper = true;
				}
				sb.deleteCharAt(i);
				i--;
			}
			else if (c == '.')
			{
				sb.delete(i, sb.length());
				break;
			}
			else if (nextUpper)
			{
				sb.setCharAt(i, Character.toUpperCase(c));
				nextUpper = false;
			}
		}

		if (sb.length() > 0)
		{
			sb.setCharAt(0, Character.toLowerCase(sb.charAt(0)));
		}

		String beanName = sb.toString();
		if (RESERVED_WORDS.contains(beanName))
		{
			return "_" + beanName;
		}

		if (beanName.length() > 0 && Character.isDigit(beanName.charAt(0)))
		{
			return "_" + beanName;
		}

		return beanName;
	}
}
